@@ -1,11 +1,7 @@
-#!/usr/bin/env python
-
-"""
-Unit tests for the doorstop.core.tree module.
-"""
+"""Unit tests for the doorstop.core.tree module."""
 
 import unittest
-from unittest.mock import patch, Mock
+from unittest.mock import patch, Mock, MagicMock
 
 import os
 import tempfile
@@ -21,12 +17,14 @@ from doorstop.core.test.test_document import MockDocument as _MockDocment
 
 
 class MockDocument(_MockDocment):  # pylint: disable=W0223,R0902,R0904
+
     """Mock Document class that is always skipped in tree placement."""
 
     skip = True
 
 
 class MockDocumentNoSkip(MockDocument):  # pylint: disable=W0223,R0902,R0904
+
     """Mock Document class that is never skipped in tree placement."""
 
     SKIP = '__disabled__'  # never skip mock Documents
@@ -34,6 +32,7 @@ class MockDocumentNoSkip(MockDocument):  # pylint: disable=W0223,R0902,R0904
 
 @patch('doorstop.core.document.Document', MockDocument)  # pylint: disable=R0904
 class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for the Tree class using strings."""  # pylint: disable=C0103
 
     @classmethod
@@ -72,9 +71,14 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
 
     def test_from_list(self):
         """Verify a tree can be created from a list."""
-        a = MockDocument(EMPTY, _prefix='A')
-        b = MockDocument(EMPTY, _prefix='B', _parent='A')
-        c = MockDocument(EMPTY, _prefix='C', _parent='B')
+        a = MockDocument(EMPTY)
+        a.prefix = 'A'
+        b = MockDocument(EMPTY)
+        b.prefix = 'B'
+        b.parent = 'A'
+        c = MockDocument(EMPTY)
+        c.prefix = 'C'
+        c.parent = 'B'
         docs = [a, b, c]
         tree = Tree.from_list(docs)
         self.assertEqual(3, len(tree))
@@ -82,24 +86,35 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
 
     def test_from_list_no_root(self):
         """Verify an error occurs when the tree has no root."""
-        a = MockDocument(EMPTY, _prefix='A', _parent='B')
-        b = MockDocument(EMPTY, _prefix='B', _parent='A')
+        a = MockDocument(EMPTY)
+        a.prefix = 'A'
+        a.parent = 'B'
+        b = MockDocument(EMPTY)
+        b.prefix = 'B'
+        b.parent = 'A'
         docs = [a, b]
         self.assertRaises(DoorstopError, Tree.from_list, docs)
 
     def test_from_list_multiple_roots(self):
         """Verify an error occurs when the tree has multiple roots."""
-        a = MockDocument(EMPTY, _prefix='A')
-        b = MockDocument(EMPTY, _prefix='B')
+        a = MockDocument(EMPTY)
+        a.prefix = 'A'
+        b = MockDocument(EMPTY)
+        b.prefix = 'B'
         docs = [a, b]
         self.assertRaises(DoorstopError, Tree.from_list, docs)
 
     def test_from_list_missing_parent(self):
         """Verify an error occurs when a node has a missing parent."""
 
-        a = MockDocument(EMPTY, _prefix='A')
-        b = MockDocument(EMPTY, _prefix='B', _parent='A')
-        c = MockDocument(EMPTY, _prefix='C', _parent='?')
+        a = MockDocument(EMPTY)
+        a.prefix = 'A'
+        b = MockDocument(EMPTY)
+        b.prefix = 'B'
+        b.parent = 'A'
+        c = MockDocument(EMPTY)
+        c.prefix = 'C'
+        c.parent = '?'
         docs = [a, b, c]
         self.assertRaises(DoorstopError, Tree.from_list, docs)
 
@@ -107,6 +122,7 @@ class TestTreeStrings(unittest.TestCase):  # pylint: disable=R0904
 @patch('doorstop.core.document.Document', MockDocument)  # pylint: disable=R0904
 @patch('doorstop.core.tree.Document', MockDocument)  # pylint: disable=R0904
 class TestTree(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for the Tree class."""  # pylint: disable=C0103
 
     def setUp(self):
@@ -155,6 +171,13 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
         with patch.object(self.tree, 'issues', mock_issues):
             self.assertFalse(self.tree.valid())
 
+    @patch('doorstop.core.document.Document.issues', Mock(return_value=[]))
+    def test_valid_hook(self):
+        """Verify a document hook can be called."""
+        mock_hook = MagicMock()
+        self.tree.valid(document_hook=mock_hook)
+        self.assertEqual(2, mock_hook.call_count)
+
     def test_new(self):
         """Verify a new document can be created on a tree."""
         self.tree.new(EMPTY, '_TEST', parent='REQ')
@@ -177,6 +200,9 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
 
     def test_add_unknown_prefix(self):
         """Verify an exception is raised for an unknown prefix (item)."""
+        # Cache miss
+        self.assertRaises(DoorstopError, self.tree.add, 'UNKNOWN')
+        # Cache hit
         self.assertRaises(DoorstopError, self.tree.add, 'UNKNOWN')
 
     @patch('doorstop.core.item.Item.delete')
@@ -227,6 +253,9 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
 
     def test_unlink_unknown_parent_prefix(self):
         """Verify an exception is raised with an unknown parent prefix."""
+        # Cache miss
+        self.assertRaises(DoorstopError, self.tree.unlink, 'req3', 'unknown1')
+        # Cache hit
         self.assertRaises(DoorstopError, self.tree.unlink, 'req3', 'unknown1')
 
     def test_unlink_unknown_parent_number(self):
@@ -252,11 +281,30 @@ class TestTree(unittest.TestCase):  # pylint: disable=R0904
 
     def test_find_item(self):
         """Verify an item can be found by exact ID."""
+        # Cache miss
         item = self.tree.find_item('req2-001')
         self.assertIsNot(None, item)
+        # Cache hit
+        item2 = self.tree.find_item('req2-001')
+        self.assertIs(item2, item)
+
+    def test_find_document(self):
+        """Verify an document can be found by prefix"""
+        # Cache miss
+        document = self.tree.find_document('req')
+        self.assertIsNot(None, document)
+        # Cache hit
+        document2 = self.tree.find_document('req')
+        self.assertIs(document2, document)
+
+    def test_load(self):
+        """Verify an a tree can be reloaded."""
+        self.tree.load()
+        self.tree.load()  # should return immediately
 
 
 class TestModule(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for the doorstop.core.tree module."""  # pylint: disable=C0103
 
     @patch('doorstop.core.vcs.find_root', Mock(return_value=EMPTY))
@@ -278,7 +326,3 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         """Verify documents can be skipped while building a tree."""
         tree = build(FILES)
         self.assertEqual(0, len(tree))
-
-
-if __name__ == '__main__':
-    unittest.main()
