@@ -1,6 +1,4 @@
-"""
-Plug-in module to simulate the storage of requirements in a repository.
-"""
+"""Plug-in module to simulate the storage of requirements in a repository."""
 
 import logging
 
@@ -8,6 +6,7 @@ from doorstop.core.vcs.base import BaseWorkingCopy
 
 
 class WorkingCopy(BaseWorkingCopy):  # pragma: no cover - integration test
+
     """Simulated working copy."""
 
     DIRECTORY = '.mockvcs'
@@ -20,4 +19,4 @@ class WorkingCopy(BaseWorkingCopy):  # pragma: no cover - integration test
 
     @property
     def ignores(self):
-        return ("*/env/*", "*/apidocs/*")
+        return ("*/env/*", "*/apidocs/*", "*/build/lib/*")
